@@ -51,24 +51,7 @@ InitOutput (ScreenInfo *pScreenInfo, int argc, char **argv)
 void
 InitInput (int argc, char **argv)
 {
-  KdKeyboardInfo *ki;
-  KdPointerInfo *pi;
-        
-  ki = KdNewKeyboard();
-  if (!ki)
-    FatalError("Couldn't create Xephyr keyboard\n");
-  ki->driver = &EphyrKeyboardDriver;
-  KdAddKeyboardDriver(&EphyrKeyboardDriver);
-  KdAddKeyboard(ki);
-
-  pi = KdNewPointer();
-  if (!pi)
-    FatalError("Couldn't create Xephyr pointer\n");
-  pi->driver = &EphyrMouseDriver;
-  KdAddPointerDriver(&EphyrMouseDriver);
-  KdAddPointer(pi);
-
-  KdInitInput();
+  KdInitInput (&EphyrMouseFuncs, &EphyrKeyboardFuncs);
 }
 
 void
